@@ -19,7 +19,6 @@
 
 """
 
-import math
 from decimal import Decimal
 from fractions import Fraction
 from functools import cached_property, lru_cache
@@ -28,6 +27,7 @@ from numbers import Real as _Real
 from typing import (
     Any,
     Generator,
+    Generic,
     List,
     Literal,
     Optional,
@@ -56,6 +56,7 @@ __all__ = ["BasedReal"]
 
 
 TBasedReal = TypeVar("TBasedReal", bound="BasedReal")
+TTypeBasedReal = TypeVar("TTypeBasedReal", bound="Type[BasedReal]")
 RadixBase = Tuple[LoopingList[int], LoopingList[int]]
 
 
@@ -497,6 +498,8 @@ class BasedReal(PreciseNumber, _Real):
         if len(left) > 0:
             rleft = left[::-1]
             for i in range(len(left)):
+                if len(rleft.strip()) == 1:
+                    break
                 separator = cls._integer_separators[-i - 1].strip().lower()
                 if separator != "":
                     split = rleft.split(separator, 1)
@@ -504,13 +507,11 @@ class BasedReal(PreciseNumber, _Real):
                         rem = split[0]
                         break
                     value, rem = split
-                else:  # pragma: no cover
+                else:
                     value = rleft[0]
                     rem = rleft[1:]
                 left_numbers.insert(0, int(value[::-1]))
                 rleft = rem.strip()
-                if len(rleft) == 1:
-                    break
             left_numbers.insert(0, int(rleft[::-1]))
 
         return cls(left_numbers, right_numbers, sign=sign)
@@ -829,7 +830,6 @@ class BasedReal(PreciseNumber, _Real):
 
         right = [0] * significant
 
-        factor = Decimal(1)
         for i in range(significant):
             factor = cls._base[1][i]
             value *= factor
@@ -1063,12 +1063,22 @@ class BasedReal(PreciseNumber, _Real):
         """other + self"""
         return self + other
 
+    def __sub__(self: TBasedReal, other) -> TBasedReal:
+        """self - other"""
+
+        return super().__sub__(other)
+
+    def __rsub__(self: TBasedReal, other) -> TBasedReal:
+        """other - self"""
+
+        return super().__rsub__(other)
+
     def _sub(self: TBasedReal, _other: PreciseNumber) -> TBasedReal:
 
         other = cast(BasedReal, _other)
         return self + -other
 
-    def __rtruediv__(self: TBasedReal, other):
+    def __rtruediv__(self: TBasedReal, other) -> TBasedReal:
         """other / self"""
         return other / float(self)
 
@@ -1171,7 +1181,7 @@ class BasedReal(PreciseNumber, _Real):
         ...
 
     @overload
-    def __mul__(self: TBasedReal, other: Unit) -> "BasedQuantity":
+    def __mul__(self: TBasedReal, other: Unit) -> "BasedQuantity[TBasedReal]":
         ...
 
     def __mul__(self: TBasedReal, other):
@@ -1193,6 +1203,16 @@ class BasedReal(PreciseNumber, _Real):
 
         return super().__mul__(other)
 
+    @overload
+    def __rmul__(  # type: ignore
+        self: TBasedReal, other: Union[float, "BasedReal"]
+    ) -> TBasedReal:
+        ...
+
+    @overload
+    def __rmul__(self: TBasedReal, other: Unit) -> "BasedQuantity[TBasedReal]":
+        ...
+
     def __rmul__(self: TBasedReal, other):
         """other * self"""
         return self * other
@@ -1226,7 +1246,7 @@ class BasedReal(PreciseNumber, _Real):
                     self.from_int(mod) >> max_significant,
                 )
 
-            fdiv = math.floor(self.decimal / other.decimal)
+            fdiv = np.floor(self.decimal / other.decimal)
             if fdiv == self.decimal / other.decimal:
                 mod = Decimal(0)
             else:
@@ -1257,11 +1277,13 @@ class BasedReal(PreciseNumber, _Real):
         return other % float(self)
 
     @overload
-    def __truediv__(self: TBasedReal, other: Number) -> TBasedReal:  # type: ignore
+    def __truediv__(  # type: ignore
+        self: TBasedReal, other: Union[float, "BasedReal"]
+    ) -> TBasedReal:
         ...
 
     @overload
-    def __truediv__(self: TBasedReal, other: Unit) -> "BasedQuantity":
+    def __truediv__(self: TBasedReal, other: Unit) -> "BasedQuantity[TBasedReal]":
         ...
 
     def __truediv__(self: TBasedReal, other):
@@ -1360,9 +1382,9 @@ class BasedReal(PreciseNumber, _Real):
             iteration = self._get_significant(self)
 
         if self >= 1:
-            res = self.from_int(int(math.sqrt(float(self))))
+            res = self.from_int(int(np.sqrt(float(self))))
         else:
-            res = self.from_float(math.sqrt(float(self)), self.significant)
+            res = self.from_float(np.sqrt(float(self)), self.significant)
             iteration = 0
 
         for _ in range(iteration):
@@ -1375,9 +1397,9 @@ class BasedReal(PreciseNumber, _Real):
         return type(self)(self.left, self.right, sign=self.sign, remainder=remainder)
 
 
-class BasedQuantity(Quantity):
+class BasedQuantity(Quantity, Generic[TBasedReal]):
 
-    value: BasedReal
+    value: TBasedReal
 
     def __new__(cls, value, unit, **kwargs):
         if (
@@ -1395,24 +1417,24 @@ class BasedQuantity(Quantity):
         self = super().__new__(cls, value, unit=unit, dtype=object, **kwargs)
         return self
 
-    def __mul__(self, other) -> "BasedQuantity":  # pragma: no cover
+    def __mul__(self, other) -> "BasedQuantity[TBasedReal]":  # pragma: no cover
         return super().__mul__(other)
 
-    def __add__(self, other) -> "BasedQuantity":  # pragma: no cover
+    def __add__(self, other) -> "BasedQuantity[TBasedReal]":  # pragma: no cover
         return super().__add__(other)
 
-    def __sub__(self, other) -> "BasedQuantity":  # pragma: no cover
+    def __sub__(self, other) -> "BasedQuantity[TBasedReal]":  # pragma: no cover
         return super().__sub__(other)
 
-    def __truediv__(self, other) -> "BasedQuantity":  # pragma: no cover
+    def __truediv__(self, other) -> "BasedQuantity[TBasedReal]":  # pragma: no cover
         return super().__truediv__(other)
 
-    def __lshift__(self, other) -> "BasedQuantity":
+    def __lshift__(self, other) -> "BasedQuantity[TBasedReal]":
         if isinstance(other, Number):
             return super(Quantity, self).__lshift__(other)
         return super().__lshift__(other)
 
-    def __rshift__(self, other) -> "BasedQuantity":
+    def __rshift__(self, other) -> "BasedQuantity[TBasedReal]":
         if isinstance(other, Number):
             return super(Quantity, self).__rshift__(other)
         return super().__rshift__(other)
@@ -1434,10 +1456,12 @@ class BasedQuantity(Quantity):
             return _new_func
         return vect(self)
 
-    def __round__(self, significant: Optional[int] = None) -> "BasedQuantity":
+    def __round__(
+        self, significant: Optional[int] = None
+    ) -> "BasedQuantity[TBasedReal]":
         return self.__getattr__("__round__")(significant)
 
-    def __abs__(self) -> "BasedQuantity":  # pragma: no cover
+    def __abs__(self) -> "BasedQuantity[TBasedReal]":  # pragma: no cover
         return self.__getattr__("__abs__")()
 
     def __quantity_subclass__(self, _):
