@@ -1,4 +1,4 @@
-from typing import Generic, TypeVar
+from typing import Generic, List, SupportsIndex, TypeVar, overload
 
 T = TypeVar("T")
 
@@ -19,6 +19,14 @@ class LoopingList(list, Generic[T]):
 
     """
 
+    @overload
+    def __getitem__(self, __i: SupportsIndex) -> T:
+        ...
+
+    @overload
+    def __getitem__(self, __s: slice) -> List[T]:
+        ...
+
     def __getitem__(self, key):
         if key >= len(self):
             return self[-1]
